<?php

/**
 * @file Workflow-ng integration for the user module 
 */

/** 
 * Implementation of hook_event_info()
 */
function user_event_info() {
  return array(
    'user_insert' => array(
      '#label' => t('User has registered'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('registered user')),
      '#redirect' => TRUE,
    ),
    'user_update' => array(
      '#label' => t('User has updated his account details'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('updated user')) + array('account_unchanged' => array('#entity' => 'user', '#label' => t('unchanged user'))),
      '#redirect' => TRUE,
    ),
    'user_view' => array(
      '#label' => t('User page has been viewed'),
      '#module' => t('User'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('viewed user')),
    ),
    'user_delete' => array(
      '#label' => t('User has been deleted'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('deleted user')),
      '#redirect' => TRUE,
    ),
    'user_login' => array(
      '#label' => t('User has logged in'),
      '#module' => t('User'),
      '#arguments' => array('account' => array('#entity' => 'user', '#label' => t('logged in user'))),
      '#redirect' => TRUE,
    ),
    'user_logout' => array(
      '#label' => t('User has logged out'),
      '#module' => t('User'),
      '#arguments' => array('account' => array('#entity' => 'user', '#label' => t('logged out user'))),
      '#redirect' => TRUE,
    ),
  );
}

/**
 * Returns the argument definition for the acting user (global $user)
 */
function workflow_ng_events_global_user_argument() {
  return array('user' => array('#entity' => 'user', '#label' => t('acting user'), '#handler' => 'workflow_ng_events_argument_global_user'));
}

/**
 * Implementation of hook_user
 * As user_user exists already, we implement workflow_ng_user()
 */
function workflow_ng_user($op, $edit, $account, $category = NULL) {
  static $account_unchanged;

  //we don't support updates for other categories than 'account'
  if ($op == 'update' && $category != 'account') {
    return;
  }
  else if ($op == 'update') {
    //save the unchanged account for the use with op after_update
    $account_unchanged = $account;
  }
  else if ($op == 'after_update') {
    workflow_ng_invoke_event('user_update', $account, NULL, $account_unchanged);
  }
  else if (in_array($op, array('insert', 'login', 'view', 'delete'))) {
    workflow_ng_invoke_event('user_'. $op, $account);
  }
}

/**
 * Returns some arguments suitable for hook_user
 */
function workflow_ng_events_hook_user_arguments($account_label) {
  return array(
    'account' => array('#entity' => 'user', '#label' => $account_label),
  ) + workflow_ng_events_global_user_argument();
}

/**
 * Gets the user account of the "acting" user - which is always the global user
 */
function workflow_ng_events_argument_global_user() {
  global $user;
  return user_load(array('uid' => $user->uid));
}


/*
 * Implementation of hook_condition_info()
 */
function user_condition_info() {
  return array(
    'workflow_ng_condition_user_comparison' => array(
      '#label' => t('Compare two users'),
      '#arguments' => array(
        'user1' => array('#entity' => 'user', '#label' => t('First user to compare')),
        'user2' => array('#entity' => 'user', '#label' => t('Second user to compare')),
      ),
      '#description' => t('Evaluates to TRUE, if both compared users are the same user account.'),
      '#module' => t('User'),
    ),
    'workflow_ng_condition_user_hasrole' => array(
      '#label' => t('User has role(s)'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('User to test')),
      ),
      '#description' => t('Whether the user has the selected role(s).'),
      '#module' => t('User'),
    ),
  );
}

/*
 * A simple user comparison
 */
function workflow_ng_condition_user_comparison($user1, $user2) {
  return $user1->uid == $user2->uid;
}

/*
 * Condition user: condition to check whether user has particular roles
 */
function workflow_ng_condition_user_hasrole($user, $settings, &$arguments, &$log) {
  $roles = $settings['roles'];
  $operation = $settings['operation'];
  
  switch ($operation) {
    case 'OR':
      foreach ($roles as $rid) {
        if (isset($user->roles[$rid])) {
          return TRUE;
        }
      }
      return FALSE;
    case 'AND':
      foreach ($roles as $rid) {
        if (!isset($user->roles[$rid])) {
          return FALSE;
        }
      }
      return TRUE;
  }
}

/*
 * Condition User: form to select roles to check for
 */
function workflow_ng_condition_user_hasrole_form($settings = array(), $argument_info) {
  $form['hasrole'] = _workflow_ng_roles($settings, FALSE);
  $form['hasrole']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Match against any or all of the selected roles'),
    '#options' => array('OR' => t('any'), 'AND' => t('all')),
    '#description' => t('If matching against all selected roles the user must have <em>all</em> the roles checked in the list above.'),
    '#default_value' => $settings['operation'],
  );
  return $form;
}

function workflow_ng_condition_user_hasrole_submit($form_id, $form_values) {
  return array(
    'roles' => array_filter(array_keys(array_filter($form_values['roles']))),
    'operation' => $form_values['operation'],
  );
}





/*
 * Implementation of hook_action_info()
 */
function user_action_info() {
  return array(
    'workflow_ng_action_user_addrole' => array(
      '#label' => t('Add user role'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('User whos roles should be changed')),
      ),
      '#module' => t('User'),
    ),
    'workflow_ng_action_user_removerole' => array(
      '#label' => t('Remove user role'),
      '#arguments' => array(
        'user' => array('#entity' => 'user', '#label' => t('User whos roles should be changed')),
      ),
      '#module' => t('User'),
    ),
  );
}

/*
 * Action user: adds roles to a particular user
 */
function workflow_ng_action_user_addrole($user, $settings, &$arguments, &$log) {
  // Get role list (minus the anonymous)
  $role_list = user_roles(1);
  $roles = $settings['roles'];
  foreach ($roles as $rid) {
    // Don't bother adding role if it's already there
    if (!in_array($rid, $user->roles)) {
      $user->roles[$rid] = $role_list[$rid];
    }
  }
  user_save($user, array('roles' => $user->roles));
}

/*
 * Action user: add user roles form
 */
function workflow_ng_action_user_addrole_form($settings = array(), $argument_info) {
  return _workflow_ng_roles($settings);
}

function workflow_ng_action_user_addrole_submit($form_id, $form_values) {
  $settings = array('roles' => array_filter(array_keys(array_filter($form_values['roles']))));
  return $settings;
}

/*
 * User Action: deletes roles from a given user
 */
function workflow_ng_action_user_removerole($user, $settings, &$arguments, &$log) {
  $roles = $settings['roles'];
  foreach ($roles as $rid) {
    // If the user has this role, delete it
    if (isset($user->roles[$rid])) {
      unset($user->roles[$rid]);
    }
  }
  user_save($user, array('roles' => $user->roles));
}

/*
 * User Action: form for selecting roles to be deleted
 */
function workflow_ng_action_user_removerole_form($settings = array(), $argument_info) {
  return _workflow_ng_roles($settings);
}

function workflow_ng_action_user_removerole_submit($form_id, $form_values) {
  $settings = array('roles' => array_filter(array_keys(array_filter($form_values['roles']))));
  return $settings;
}

/*
 * Helper function to create role form
 */
function _workflow_ng_roles($settings = array(), $roles_only = TRUE) {
  $roles = user_roles($roles_only);
  if ($roles_only) {
    unset($roles[DRUPAL_AUTHENTICATED_RID]);
  }

  $form = array();
  $form['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Select role(s)'),
    '#options' => $roles,
    '#default_value' => $settings['roles'],
    '#required' => TRUE,
  );
  return $form;
}
